from shuttlis.time import time_now

from ats_sdk.domain import TranslatableMessages
from ats_sdk.translation import translate, fetch_translatable_strings


def test_fetch_translatable_strings_in_simple_dict():
    class FakeMessages(TranslatableMessages):
        NAME = "cb.name"
        CITY = "cb.city"

    resource = {"name": FakeMessages.NAME, "city": FakeMessages.CITY}

    translatable_strings = set(fetch_translatable_strings(resource))

    assert {"cb.name", "cb.city"} == translatable_strings


def test_fetch_translatable_strings_in_nested_dict():
    class FakeMessages(TranslatableMessages):
        NAME = "cb.name"
        CITY = "cb.city"

    resource = {"name": FakeMessages.NAME, "data": {"city": FakeMessages.CITY}}

    translatable_strings = set(fetch_translatable_strings(resource))

    assert {"cb.name", "cb.city"} == translatable_strings


def test_fetch_translatable_strings_in_nested_dict_with_list():
    class FakeMessages(TranslatableMessages):
        NAME = "cb.name"
        CITY = "cb.city"

    resource = {
        "name": FakeMessages.NAME,
        "data": [{"city": FakeMessages.CITY}, {"name": FakeMessages.NAME}],
    }

    translatable_strings = set(fetch_translatable_strings(resource))

    assert {"cb.name", "cb.city"} == translatable_strings


def test_fetch_translatable_strings_in_nested_dict_with_other_types():
    class FakeMessages(TranslatableMessages):
        NAME = "cb.name"
        CITY = "cb.city"

    resource = {
        "name": FakeMessages.NAME,
        "house": "random",
        "age": 10,
        "dob": time_now().date(),
        "data": [{"city": FakeMessages.CITY}, {"name": FakeMessages.NAME}],
    }

    translatable_strings = set(fetch_translatable_strings(resource))

    assert {"cb.name", "cb.city"} == translatable_strings


def test_translate_with_simple_dict():
    class FakeMessages(TranslatableMessages):
        NAME = "cb.name"
        CITY = "cb.city"

    resource = {"name": FakeMessages.NAME, "city": FakeMessages.CITY}
    translated_strings = {"cb.name": "name in en", "cb.city": "city in en"}

    translated_resource = translate(
        resource=resource, translated_strings=translated_strings
    )

    assert {"name": "name in en", "city": "city in en"} == translated_resource


def test_translate_with_nested_dict():
    class FakeMessages(TranslatableMessages):
        NAME = "cb.name"
        CITY = "cb.city"

    resource = {"name": FakeMessages.NAME, "data": {"city": FakeMessages.CITY}}
    translated_strings = {"cb.name": "name in en", "cb.city": "city in en"}

    translated_resource = translate(
        resource=resource, translated_strings=translated_strings
    )

    assert {"name": "name in en", "data": {"city": "city in en"}} == translated_resource


def test_translate_with_nested_dict_with_list():
    class FakeMessages(TranslatableMessages):
        NAME = "cb.name"
        CITY = "cb.city"

    resource = {
        "name": FakeMessages.NAME,
        "data": [{"city": FakeMessages.CITY}, {"name": FakeMessages.NAME}],
    }
    translated_strings = {"cb.name": "name in en", "cb.city": "city in en"}

    translated_resource = translate(
        resource=resource, translated_strings=translated_strings
    )

    assert {
        "name": "name in en",
        "data": [{"city": "city in en"}, {"name": "name in en"}],
    } == translated_resource


def test_translate_with_nested_dict_with_other_types():
    class FakeMessages(TranslatableMessages):
        NAME = "cb.name"
        CITY = "cb.city"

    resource = {
        "name": FakeMessages.NAME,
        "house": "random",
        "age": 10,
        "dob": time_now().date(),
        "data": [{"city": FakeMessages.CITY}, {"name": FakeMessages.NAME}],
    }

    translated_strings = {"cb.name": "name in en", "cb.city": "city in en"}

    translated_resource = translate(
        resource=resource, translated_strings=translated_strings
    )

    assert {
        "name": "name in en",
        "house": "random",
        "age": 10,
        "dob": time_now().date(),
        "data": [{"city": "city in en"}, {"name": "name in en"}],
    } == translated_resource



from typing import Dict

from ats_sdk.domain import TranslatableMessages


def fetch_translatable_strings(resource: Dict):
    if isinstance(resource, TranslatableMessages):
        yield str(resource)
    elif isinstance(resource, list):
        for val in resource:
            yield from fetch_translatable_strings(val)
    elif isinstance(resource, dict):
        for val in resource.values():
            yield from fetch_translatable_strings(val)


def translate(resource: Dict, translated_strings: Dict):
    def _translate(obj: dict):
        if isinstance(obj, TranslatableMessages):
            return translated_strings.get(str(obj))
        if isinstance(obj, list):
            return [_translate(val) for val in obj]
        if isinstance(obj, dict):
            return {k: _translate(v) for k, v in obj.items()}

        return obj

    return _translate(resource)



from http import HTTPStatus

import pytest
from responses import mock
from shuttlis.time import time_now

from ats_sdk.domain import TranslatableMessages
from ats_sdk.error import ClientError
from ats_sdk.services import AlternateTextService

fake_ats_url = "https://fake_ats_url"


@mock.activate
def test_translate_and_serialize_successful():
    mock.add(
        mock.GET,
        f"{fake_ats_url}/api/v1/translate/static",
        status=HTTPStatus.OK,
        json={"data": {"cb.name": "name in en", "cb.city": "city in en"}},
    )

    class FakeMessages(TranslatableMessages):
        NAME = "cb.name"
        CITY = "cb.city"

    dob = time_now().date()
    resource = {
        "name": FakeMessages.NAME,
        "age": 10,
        "data": {
            "hello": "world",
            "dob": dob,
            "name": FakeMessages.NAME,
            "city": FakeMessages.CITY,
        },
    }

    translated_resource = AlternateTextService(
        url=fake_ats_url
    ).translate_and_serialize(resource=resource, locale="en")

    assert {
        "name": "name in en",
        "age": 10,
        "data": {
            "hello": "world",
            "dob": dob.isoformat(),
            "name": "name in en",
            "city": "city in en",
        },
    } == translated_resource


@mock.activate
def test_translate_and_serialize_raises_client_error_if_translation_fails():
    mock.add(
        mock.GET,
        f"{fake_ats_url}/api/v1/translate/static",
        status=HTTPStatus.INTERNAL_SERVER_ERROR,
    )

    class FakeMessages(TranslatableMessages):
        NAME = "cb.name"
        CITY = "cb.city"

    dob = time_now().date()
    resource = {
        "name": FakeMessages.NAME,
        "age": 10,
        "data": {
            "hello": "world",
            "dob": dob,
            "name": FakeMessages.NAME,
            "city": FakeMessages.CITY,
        },
    }

    with pytest.raises(ClientError):
        AlternateTextService(url=fake_ats_url).translate_and_serialize(
            resource=resource, locale="en"
        )



from http import HTTPStatus
from typing import List, Dict

import requests
from shuttlis.serialization import serialize

from ats_sdk.error import ClientError
from ats_sdk.translation import fetch_translatable_strings, translate


class AlternateTextService:
    def __init__(self, url: str):
        self._url = url

    def translate_and_serialize(self, resource: Dict, locale: str):
        translatable_strings = set(fetch_translatable_strings(resource=resource))

        translated_strings = self.get_static_translation(
            locale=locale, keys=list(translatable_strings)
        )

        return serialize(
            translate(resource=resource, translated_strings=translated_strings)
        )

    def get_static_translation(self, locale: str, keys: List[str]):
        response = requests.get(
            f"{self._url}/api/v1/translate/static",
            params={"locale": locale, "keys": ",".join(keys)},
        )
        if response.status_code != HTTPStatus.OK:
            raise ClientError(response.text)

        return response.json()["data"]



import setuptools

setuptools.setup(
    name="ats_sdk",
    version="0.0.1",
    description="SDK for alternate text service",
    url="https://github.com/shuttl-tech/ats_sdk",
    author="Shuttl",
    author_email="pratik.singh@shuttl.com",
    license="MIT",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: MIT License",
    ],
    install_requires=["requests", "pyshuttlis"],
    extras_require={
        "test": [
            "pytest",
            "pytest-runner",
            "pytest-cov",
            "pytest-pep8",
            "responses",
            "bumpversion",
        ]
    },
)
